@@ -610,15 +610,24 @@ Container <- R6::R6Class(
       }
 
       if (is.character(loadFrom)) {
-        namesplit <- strsplit(loadFrom, "\\.")
-        ext <- utils::tail(unlist(namesplit), 1)
-        if (ext != "gdx") {
-          stop("check filename extension, must be .gdx\n")
-        }
         loadFrom <- R.utils::getAbsolutePath(path.expand(loadFrom))
+
+        ext = tools::file_ext(loadFrom)
         if (!file.exists(loadFrom)) {
-          stop(paste0("File ", loadFrom, " doesn't exist\n"))
+          if (ext != "") {
+            stop(paste0("File ", loadFrom, " doesn't exist\n"))
+          }
+          else {
+            # try if adding .gdx returns in file exists
+            if (file.exists(paste0(loadFrom, ".gdx"))) {
+              loadFrom = paste0(loadFrom, ".gdx")
+            }
+            else {
+              stop(paste0("File ", loadFrom, " doesn't exist\n"))
+            }
+          }
         }
+
         private$.gdxRead(loadFrom, symbols, records)
       } else if (inherits(loadFrom, "Container")) {
         private$.containerRead(loadFrom, symbols, records)
@@ -1000,12 +1009,6 @@ Container <- R6::R6Class(
       if (!is.character(writeTo)) {
         stop("The argument writeTo must be of type character\n")
       } else {
-        namesplit <- strsplit(writeTo, "\\.")
-        ext <- utils::tail(unlist(namesplit), 1)
-        if (ext != "gdx") {
-          stop("check filename extension, must be .gdx\n")
-        }
-
         writeTo <- R.utils::getAbsolutePath(path.expand(writeTo))
       }
 
