@@ -1,7 +1,7 @@
 // GAMS - General Algebraic Modeling System R API
 //
-// Copyright (c) 2017-2024 GAMS Software GmbH <support@gams.com>
-// Copyright (c) 2017-2024 GAMS Development Corp. <support@gams.com>
+// Copyright (c) 2017-2025 GAMS Software GmbH <support@gams.com>
+// Copyright (c) 2017-2025 GAMS Development Corp. <support@gams.com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 
 #include "gclgms.h"
 using namespace Rcpp;
-#include "gdx.h"
+#include "gdx.hpp"
 
 class gt_gdx
 {
